﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Collections.Generic;
using com.cruxomatic.DTOs;
using com.cruxomatic.views.Administration;
using com.cruxomatic.controllers.Administration;
using com.cruxomatic.views.Enums;
using com.cruxomatic.DTOs.Configuration;
using com.cruxomatic.configuration;

public partial class Administration_ManageLocations : BaseView, IManageLocations 
{
    ManageLocationsController _controller = null;
    protected FullGridPager fullGridPager;
    protected int MaxVisible = 0;

    protected void Page_Load(object sender, EventArgs e)
    {
        _controller = new ManageLocationsController(this);
        MaxVisible = Convert.ToInt32(CruxConfigurationManager.GetApplicationCustomSetting("MaximumPageCount").Value);
        grdLocations.DataBound += new EventHandler(GridDataBound);
        _controller.InitView(IsPostBack);
        if (!IsPostBack)
        {
            #region Site Map
            SetSiteMap();
            #endregion
            SetPageSize();
            Page.DataBind();
        }
        if (IsPostBack)
        {
            fullGridPager = new FullGridPager(grdLocations, MaxVisible, "Page", "of");
            fullGridPager.CreateCustomPager(grdLocations.BottomPagerRow);
        }
    }

    #region Site Map
    private void SetSiteMap()
    {
        string query = Request.Url.Query;
        UrlTreeNodeItems.ManageLocation = UrlTreeNodeItems.CreateNode("Manage Location",
                                                                          "/Administration/ManageLocations.aspx" + query,
                                                                          "ManageLocation",
                                                                          UrlTreeNodeItems.ManageMyOrganization);
        UrlTreeNodeItems.SetSiteNavigation(PageSiteMap, UrlTreeNodeItems.ManageLocation);
    }
    #endregion

    #region Paging

    protected void GridDataBound(object sender, EventArgs e)
    {
        if (fullGridPager == null)
        {
            fullGridPager = new FullGridPager(grdLocations, MaxVisible, "Page", "of");
        }
        fullGridPager.CreateCustomPager(grdLocations.BottomPagerRow);
        fullGridPager.PageGroups(grdLocations.BottomPagerRow);
    }

    private void SetPageSize()
    {
        int PagerSize = Convert.ToInt32(CruxConfigurationManager.GetApplicationCustomSetting("PagerSize").Value);
        grdLocations.PageSize = PagerSize;
    }

    protected void PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        grdLocations.PageIndex = e.NewPageIndex;
        _controller.SwitchToListMode();
    }

    #endregion

    public void grdLocations_RowCommand(object sender, CommandEventArgs e)
    {
        RowCommands rowCommand = (RowCommands)Enum.Parse(typeof(RowCommands), e.CommandName);
        if (rowCommand == RowCommands.EditItem)
        {
            string locationId = e.CommandArgument.ToString();
            _controller.OpenLocation(int.Parse(locationId));
            UrlTreeNodeItems.SetSiteNavigation(PageSiteMap, UrlTreeNodeItems.CreateLeafUrlTreeNode("Edit - " + CurrentLocation.LocationName, UrlTreeNodeItems.ManageLocation));
        }
        else if (rowCommand == RowCommands.DeleteItem)
        {
            string locationId = e.CommandArgument.ToString();
            _controller.DeleteLocation(int.Parse(locationId));
        }

        else if (rowCommand == RowCommands.OpenView)
        {
            string locationId = e.CommandArgument.ToString();
            _controller.OpenSubLocationView(int.Parse(locationId));
        }
    }

    protected void grdLocations_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            ImageButton deleteButton = (ImageButton)e.Row.FindControl("btnItemDelete");
            string ReplaceConfirmation = new UILiterals().DeleteLocationConfirmation.Replace("{0}",
                DataBinder.Eval(e.Row.DataItem, "LocationName").ToString());
            deleteButton.Attributes.Add("onclick", "javascript:return "
                + "confirm('"+ ReplaceConfirmation + "')");
        }
    }

   
    #region IManageLocations Members

    public List<LocationInformation> Locations
    {
        get
        {
            return (List<LocationInformation>) grdLocations.DataSource;
        }
        set
        {
            grdLocations.DataSource = value;
            grdLocations.DataBind();
        }
    }

    public LocationInformation CurrentLocation
    {
        get
        {
            LocationInformation locationInformation = new LocationInformation();
            locationInformation.LocationID = int.Parse(lblLocationIDValue.Text);
            locationInformation.LocationName = txtLocationName.Text;
            locationInformation.LocationAddress = txtLocationAddress.Text;
            locationInformation.LocationComments = txtLocationComments.Text;
            locationInformation.CompanyID = CompanyID;
            locationInformation.CompanyName = 
                               _controller.GetCompanyForLocation(CompanyID).CompanyName;
            return locationInformation;
        }
        set
        {
            lblLocationIDValue.Text = value.LocationID.ToString();
            txtLocationName.Text = value.LocationName;
            txtLocationAddress.Text = value.LocationAddress;
            txtLocationComments.Text = value.LocationComments;
        }
    }

    public FormMode FormMode
    {
        get
        {
            if (pnlList.Visible == true)
                return FormMode.List;
            else if (pnlAddEdit.Visible == true)
                return FormMode.AddEdit;
            else
                throw UIExceptions.InvalidFormModeException;
        }
        set
        {
            if (value == FormMode.List)
            {
                pnlList.Visible = true;
                pnlAddEdit.Visible = false;
            }
            else if (value == FormMode.AddEdit)
            {
                pnlList.Visible = false;
                pnlAddEdit.Visible = true;
            }
            else
            {
                throw UIExceptions.InvalidFormModeException;
            }
        }
    }

    #endregion
    protected void btnSave_Click(object sender, EventArgs e)
    {
        _controller.SaveLocation();
        #region Site Map
        if (FormMode == FormMode.List)
            SetSiteMap();
        #endregion
    }
    protected void btnCancel_Click(object sender, EventArgs e)
    {
        _controller.CancelAddEdit();
        #region Site Map
        SetSiteMap();
        #endregion
    }
    protected void btnAddNewLocation_Click(object sender, EventArgs e)
    {
        _controller.OpenLocation(0);
        #region Site Map
        UrlTreeNodeItems.SetSiteNavigation(PageSiteMap, UrlTreeNodeItems.CreateLeafUrlTreeNode("Add", UrlTreeNodeItems.ManageLocation));
        #endregion
    }

    protected void btnBack_Click(object sender, EventArgs e)
    {
        _controller.RedirectToPreviousPage();
    }

    #region IManageLocations Members


    public ErrorInformation ErrorDetails
    {
        get
        {
            return pnlError.ErrorInformation;
        }
        set
        {
            pnlError.ErrorInformation = value;
        }
    }

    #endregion

    #region IManageLocations Members


    public int CompanyID
    {
        get { return int.Parse(Request.Params["id"].ToString()); }
    }


    #endregion
}
