﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using com.cruxomatic.BusinessObjects;
using com.cruxomatic.BusinessObjects.DatabaseQueries;
using com.cruxomatic.DTOs;
using com.cruxomatic.configuration;
using com.cruxomatic.exceptions;
using com.cruxomatic.logging;
using com.cruxomatic.logging.Enums;
using com.cruxomatic.notifications;
using com.cruxomatic.security.EncryptionAndHashing;
using com.cruxomatic.security.EncryptionAndHashing.Enums;

namespace com.cruxomatic.BusinessFacades
{
    public class ForgotPasswordFacade
    {
        private KeyValuePair<string, string> GenerateRandomPassword(PasswordStorageType passwordStorageType)
        {
            //TODO: Improove Password Generation Mechanism. First Six Characters Of Guid Are A Neat Idea
            //but find out how the rest of the world does it. .NET is supposed to have some API for this
            //becuase the membership providers renegrate forgotten passwords. 

            Guid randomKey = Guid.NewGuid();
            string randomPassword = randomKey.ToString().Substring(0, 6);
            string randomHashedPassword = randomPassword;
            if (passwordStorageType == PasswordStorageType.hashed)
                randomHashedPassword = Hasher.GetHash(randomPassword);
            KeyValuePair<string, string> NormalAndHashedPassword = new KeyValuePair<string, string>(randomPassword, randomHashedPassword);
            return NormalAndHashedPassword;
        }

        public void ResetPassword(string userName, PasswordStorageType passwordStorageType, int savedPasswordCount, bool isFeatureSupported)
        {
            PasswordFacade passwordFacade = new PasswordFacade();
            KeyValuePair<string, string> RandomAndHashedPassword = GenerateRandomPassword(passwordStorageType);
            string RandomPassword = RandomAndHashedPassword.Key;
            Queries query = new Queries();
            User dbUser = query.GetUser(userName);
            dbUser.Password = RandomAndHashedPassword.Value;
            dbUser.PasswordChangedOn = DateTime.Now;
            dbUser.IsResetPasswordNeeded = true;
            if(isFeatureSupported)
             passwordFacade.SavePassword(dbUser, savedPasswordCount, query);
            query.SubmitChanges();
            try
            {
                SendForgotPasswordNotificationToUser(dbUser.FirstName + " " + dbUser.LastName,
                                                                    RandomPassword, dbUser.UserName);
            }
            catch (Exception ex)
            {
                ApplicationLogger.LogApplicationMessage(ex.Message, ApplicationLogType.Notification, ex.StackTrace);
                throw CruxExceptionManager.CreateNewNotificationException("701");
            }
        }

        public void SendForgotPasswordNotificationToUser(string fullName, string RandomPassword, string userName)
        {
            List<KeyValuePair<string, string>> TagsAndValues = new List<KeyValuePair<string, string>>();
            TagsAndValues.Add(new KeyValuePair<string, string>("FullName", fullName));
            TagsAndValues.Add(new KeyValuePair<string, string>("UserName", userName));
            TagsAndValues.Add(new KeyValuePair<string, string>("Password", RandomPassword));

            List<string> recepients = new List<string>();
            recepients.Add(userName);

            MailMan mailman = new MailMan(CruxConfigurationManager.GetFullConifgurationSettings());
            mailman.SendMail("ForgotPassword", TagsAndValues, recepients);
        }

        public List<SecurityQuestionInformation> GetAllSecurityQuestions()
        {
            Queries query = new Queries();
            IEnumerable<SecurityQuestionInformation> questions = from ques in query.GetAllSecurityQuestions()
                                                                 select ClassConverter.CopySecurityQuestionToSecurityQuestionInformation(ques);
            return questions.ToList();
        }

        public bool ValidateDetails(string userName, int securityQuesId, string ans)
        {
            Queries query = new Queries();
            User dbUser = query.GetUser(userName);
            if (dbUser.SecurityQuestionID == securityQuesId && dbUser.SecurityAnswer.ToLower() == ans.ToLower())
                return true;
            else
                return false;
        }
    }
}
